/* 
 * Interp.java --
 *
 *	This file contains the implementation of the Java API
 *	for creating and controlling Tcl interpreters.
 *
 * Copyright (c) 1995 by Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) Interp.java 1.3 95/12/19 14:34:40
 */

package tcl;
import java.util.Vector;

/**
 * A Tcl interpreter.
 */

public class Interp {

    /** Result code indicating all is fine. */
    static final int TCL_OK = 0;
    /** Result code indicating an error condition. */
    static final int TCL_ERROR = 1;
    /** Result code indicating a return request. */
    static final int TCL_RETURN = 2;
    /** Result code indicating a break request. */
    static final int TCL_BREAK = 3;
    /** Result code indicating a continue request. */
    static final int TCL_CONTINUE = 4;

    /*
     * Flags passed to doOneEvent.  These values may be or'ed together.
     */

    /**
     * Flag for doOneEvent indicating that the method should not wait
     * if no events are ready.
     */
    public static final int TCL_DONT_WAIT = (1<<1);
    /** Flag for doOneEvent to process window events. */
    public static final int TCL_WINDOW_EVENTS = (1<<2);
    /** Flag for doOneEvent to process file events. */
    public static final int TCL_FILE_EVENTS = (1<<3);
    /** Flag for doOneEvent to process timer events. */
    public static final int TCL_TIMER_EVENTS = (1<<4);
    /** Flag for doOneEvent to process idle events. */
    public static final int TCL_IDLE_EVENTS = (1<<5);
    /** Flag value for doOneEvent selecting all event types for handling. */
    public static final int TCL_ALL_EVENTS = (~TCL_DONT_WAIT);

    /**
     * Creates a new interpreter.
     */
    public Interp () {
	commands = new Vector();
	create();
    }

    /**
     * Evals a Tcl script.
     * Results in a call to eval(s, false).
     * @param s the Tcl script to be evaluated.
     * @return the result of the script
     * @exception Tcl_ErrorException signals Tcl-style error conditions.
     * @see Tcl_ErrorException
     */
    public Value
    eval(String s) throws Tcl_ErrorException
    {
	// Only error exceptions should occur here, because the false
	// value for allowExceptions will ensure that control exceptions
	// are translated to errors before the eval returns.  However,
	// Java doesn't know that, so we catch them to keep Java happy.
	try {
	    return eval(s, false);
	}
	catch (Tcl_BreakException e) {
	    // Even though this shouldn't happen, since we have to catch
	    // it, we may as well do the right thing.
	    throw new Tcl_ErrorException("invoked \"break\" outside of "
					 + "a loop");
	}
	catch (Tcl_ContinueException e) {
	    throw new Tcl_ErrorException("invoked \"continue\" outside of "
					 + "a loop");
	}
	catch (Tcl_ReturnException e) {
	    return e.getResult();
	}
	catch (Tcl_ResultCodeException e) {
	    // This is either a Tcl_UserException or some other subclass
	    // of Tcl_ResultCodeException that we don't know about.
	    throw new Tcl_ErrorException("command returned bad code: "
                                         + e.getCode());
	}
    }

    /**
     * Evals a Tcl script.
     * @param s the Tcl script to be evaluated.
     * @param allowExceptions determines whether Tcl-style non-error
     * exceptions (such as return, break, continue, and user-defined
     * exceptions) should be returned.  If this parameter is false, such
     * exceptions are transformed to error conditions.
     * @return the result of the script
     * @exception Tcl_ResultCodeException signals Tcl-style exception
     * conditions, including return, break, and continue conditions in
     * addition to errors and user-defined exceptions.
     * @see Tcl_ErrorException
     * @see Tcl_UserException
     * @see Tcl_BreakException
     * @see Tcl_ContinueException
     * @see Tcl_ReturnException
     */
    public Value
    eval(String s, boolean allowExceptions) throws Tcl_ResultCodeException
    {
	Result r = native_eval(s, allowExceptions);
	switch (r.returnCode) {
	    case TCL_OK:
		return r.value;
		// break
	    case TCL_ERROR:
		throw new Tcl_ErrorException(r.value);
		// break
	    case TCL_RETURN:
		throw new Tcl_ReturnException(r.value);
		// break
	    case TCL_BREAK:
		throw new Tcl_BreakException();
		// break
	    case TCL_CONTINUE:
		throw new Tcl_ContinueException();
		// break
	    default:
		throw new Tcl_UserException(r.value, r.returnCode);
		// break
	}
    }
    
    /**
     * Internal method for performing native Tcl_eval call.
     * @param s the Tcl script to be evaluated.
     * @param allowExceptions determines whether Tcl-style non-error
     * exceptions (such as return, break, continue, and user-defined
     * exceptions) should be returned.  If this parameter is false, such
     * exceptions are transformed to error conditions.
     * @return the result of the script, Tcl-style: a (return code, result
     * string) pair.
     */
    private native synchronized Result native_eval(String s,
						   boolean allowExceptions);

    /**
     * Create a new Tcl command with the given implementation object.
     * @param s the command name
     * @param obj the command implementation object
     */
    public native synchronized void createCommand(String s, Command obj);

    /**
     * Process one pending event off the event queue.
     * @param flags indicates which kinds of events should be processed,
     * and whether the method should wait if no events are currently
     * available.  Several flag constants are defined as constants in
     * this class; they can be ORed together to produce this parameter.
     * @return the number of events processed (0 or 1).
     * @see #TCL_DONT_WAIT
     * @see #TCL_WINDOW_EVENTS
     * @see #TCL_FILE_EVENTS
     * @see #TCL_TIMER_EVENTS
     * @see #TCL_IDLE_EVENTS
     * @see #TCL_ALL_EVENTS
     */
    public static native int doOneEvent(int flags);

    /*
     * Private initialization method.
     */
    private native void create();

    /*
     * Private implementation variables.
     */
    private Vector commands;	// Array of Command objects.
    private int interp;		// Interpreter handle.
}

/**
 * A result, Tcl-style: a (return code, result string) pair.
 */

class Result {

    /**
     * Creates a new Result.
     * @param returnCode the return code
     * @param value the result string
     */
    Result(int returnCode, String value)
    {
	this.returnCode = returnCode;
	this.value = new Value(value);
    }

    /** the return code */
    int returnCode;

    /** the result string */
    Value value;
}
