/* 
 * Tcl_UserException.java --
 *
 *	This file defines the class Tcl_UserException, which is used
 *      to signal Tcl user-defined exceptions within Java code.
 *
 * Copyright (c) 1996 by Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS:
 */

package tcl;

/**
 * An exception for user-defined Tcl result codes.
 *
 * This class is derived from Tcl_ErrorException so that it can easily
 * be caught and handled as an error by code which is not expecting any
 * particular user-defined result codes.  I believe that the code in
 * the Tcl interpreter treats unexpected user-defined codes as errors.
 * Since the default action for errors is to just add some tracing info
 * and propagate the result code upward, that's what you want: if it
 * eventually reaches something that is expecting the user code, fine,
 * otherwise, it will be caught by a "catch" command or reported as an
 * error.
 *
 * Code that wants to catch <em>only</em> genuine instances of
 * Tcl_ErrorException can do so by catching Tcl_UserException before
 * the clause for Tcl_ErrorException (and promptly re-throwing the
 * Tcl_UserException).
 *
 * @version     1.0, 23 August 1996
 * @author      Glenn Vanderburg
 */

public
class Tcl_UserException extends Tcl_ErrorException 
{
    /**
     * The result code for this exception.
     */
    private int code;
    
    /**
     * Constructs a new instance.
     * @param code the desired integer result code (must not be one of
     * the predefined Tcl result codes in Tcl_ResultCodeException)
     * @param descrip A textual description of the condition
     * @exception IllegalArgumentException thrown if the code is not one
     * of those reserved for user-defined exceptions.
     * @see Tcl_ResultCodeException
     */
    public
    Tcl_UserException(Value descrip, int code) 
    {
	// This must come first ...
	super("A Tcl user exception has occurred", descrip);

	// ... but we do some validation and abort if all is not well.
	if (    code == Interp.TCL_OK
	        || code == Interp.TCL_ERROR
	        || code == Interp.TCL_RETURN
	        || code == Interp.TCL_BREAK
		|| code == Interp.TCL_CONTINUE) {
	    // Our caller is attempting to falsify one of the other
	    // exception types.
	    throw new IllegalArgumentException();
	} else {
	    this.code = code;
	}
    }

    /**
     * Constructs a new instance.
     * @param descrip A textual description of the condition
     * @param code the desired integer result code (must not be one of
     * the predefined Tcl result codes in Tcl_ResultCodeException)
     * @exception IllegalArgumentException thrown if the code is not one
     * of those reserved for user-defined exceptions.
     * @see Tcl_ResultCodeException
     */
    public
    Tcl_UserException(String descrip, int code)
    {
        this(new Value(descrip), code);
    }

    /**
     * The default constructor is not allowed.
     */
    private
    Tcl_UserException()
    {
	super(new Value("Something's wrong!"));
    }

    /**
     * Returns the Tcl result code for this exception
     * @return the Tcl result code
     */
    public final int
    getCode()
    {
	return code;
    }
}
